%{
#include "token.h"
#include <stdio.h>
#undef YY_INPUT
#define YY_INPUT(b, r, ms) (r = minput(b, ms))
extern int state; /* Set this. It's important */
extern long int lcount; /* Irrevelant */
extern char *ptr;
int warn = 0;
char *spare; /* For tricky manipulation in rules */
long int yyval;
%}

%x CMT

%%
<CMT>\n			|
<CMT>\#			BEGIN INITIAL; /* End of comment, then */
<CMT>.			;					/* Ignore anything in a comment */
\"[^"]*\"		{ return TEXT; } ; /* I *know* this is nonstandard */
\`[^`]*\`		{ return TICK; } ; /* I *know* this is nonstandard */
~\/[^/]*\/		{ return RE; } ; /* Regular expression */
\#[^ \n]*		BEGIN CMT; /* Beginning of a comment */
\]					{ return CBRACE; }; /* Opening brace */
\[					{ return OBRACE; }; /* Closing brace */
0					{ yyval = 0; return NUM; } ; /* Special case */
-?[1-9][0-9]*	{ yyval = atoi(yytext); return NUM; } ; /* First digit can't be 0 */
-?([0-9]*\.[0-9]+) { return FLOAT; } /* Convert later */
0x[0-9A-F]+		{ yyval = unhex(yytext); return NUM; } ; /* Compile hex normally */
0[0-7]+			{ yyval = unoct(yytext); return NUM; } ; /* Same with octal */
\+					{ return PLUS; } ; /* Plus */
\-					{ return MINUS; } ; /* Minus */
\*					{ return TIMES; } ; /* Times */
\/					{ return OVER; } ; /* Divided by */
%					{ return MOD; } ; /* Take modulus */
\^					{ return TO; } ; /* Raise to power */
&[^\t \n]+		{ return SUBC; } ; /* Subroutine */
\/[^\t \n]+		{ return EXTC; } ; /* Extension call */
:[^\t \n]+		{ return VLOOK; } ; /* Variable lookup */
\|[^\t \n]+		{ return RLOOK; } ; /* Reference lookup */
\}[^\t \n]+		{ return VSET; } ; /* Var set */
\)[^\t \n]+		{ return ISET; } ; /* Ref set */
\>[^\t \n]+		{ return FSET; } ; /* Set fdesc */
[^\t \n]+\>		{ return FGET; } ; /* Retrieve desc */
[^\t \n]+:		{ return GLABEL; } ; /* Label for goto */
s\/[^/]+\/[^/\n]*\/g?	{ return QSUB; } ; /* Quick syntax for subs */
m\/[^/\n]+\/				{ return QMATCH; } ; /* Quick match */
goto				{ return GOTO; } ; /* Goto label */
sqrt				{ return SQRT; } ; /* Square root */
sin				{ return SIN; } ; /* Returns sine */
launch			{ return LLAMA; } ; /* Launch an extension */
cos				{ return COS; } ; /* Returns cosine */
tan				{ return TAN; } ; /* Returns tangent */
log10				{ return LOG10; } ; /* Base-10 log */
loge				{ return LOGE; } ; /* Base-e log */
fork				{ return FORK; } ; /* Fork new instance */
iref				{ return IREF; } ; /* Make an int reference */
ider				{ return IDER; } ; /* Follow an int reference */
sref				{ return SREF; } ; /* Make a string reference */
sub				{ return SUB; } ; /* Start a subroutine */
sder				{ return SDER; } ; /* Follow a string reference */
vref				{ return VREF; } ; /* Return pointer to variable */
stoi				|
refs				{ return REFS; } ; /* Store a reference */
geti				|
refg				{ return REFG; } ; /* Fetch a reference */
sort				{ return SORT; } ; /* Sort numbers on stack */
sortS				{ return sSORT; } ; /* Sort strings */
hashl				{ return HASHL; } ; /* Do a hash lookup */
hasha				{ return HASHA; } ; /* Add to a hash */
llnew				|						  /* Note hashes == ll's */
hashn				{ return HASHN; } ; /* Create a new hash table */
hashk				{ return HASHK; } ; /* Give the keys of a hash table */
llpush			|
lladd				{ return LLADD; } ; /* Add to ll */
llat				{ return LLAT; } ; /* Add to ll at position */
lllength			{ return LLLENGTH; } ; /* Return size of ll */
lldel				{ return LLDEL; } ; /* Delete item from ll */
lldup				{ return LLDUP; } ; /* Duplicate ll */
llcat				{ return LLCAT; } ; /* Concatenate ll */
llunroll			{ return LLUNROLL; } ; /* Unroll a list */
llshift			{ return LLUSHIFT; } ; /* Shift an element in queue */
llget				|
lllook			{ return LLLOOK; } ; /* Lookup linked list */
llset				{ return LLSET; } ; /* Set an element of an ll */
llrev				{ return LLREV; } ; /* Reverse linked list */
llpop				{ return LLPOP; } ; /* Pop off ll */
rval				{ return RVAL; } ; /* Return value */
rand				{ return RAND; } ; /* Random number */
sleep				{ return SLEEP; } ; /* Sleep n seconds */
alarm				{ return ALARM; } ; /* Deliver SIGALRM in n seconds */
srand				{ return SRAND; } ; /* Set random number seed */
Use				|
type				{ return USE; } ; /* Include routines from a file */
sdesc				{ return SFILE; } ; /* Set descriptor */
gdesc				{ return GFILE; } ; /* Get descriptor */
ext				{ return EXT; } ; /* Call EXTernal EXTension */
time				{ return TIME; } ; /* Get time in UNIX format */
atime				{ return ATIME; } ; /* Get time in ASCII format */
rcc				{ return RCC; } ; /* Return counter*/
stc				{ return STC; } ; /* Store counter */
if					{ return IF; } ; /* A genuine if */
else				{ return ELSE; } ; /* Otherwise */
while				{ return WHILE; } ; /* While loop */
break				{ return BREAK; } ; /* Break loop */
forever			{ return FOREVER; } ; /* As implies */
forf				{ return FORF; } ; /* For file */
repeat			{ return REPEAT; } ; /* Repeat statement x times */
call				{ return CALL; } ; /* Call a subroutine by number */
-[A-Za-z]		{ return TEST; } ; /* Like shell -x commands */
out				{ return OUT; } ; /* Output number */
i->f				{ return D2F; } ; /* Decimal to float */
f->i				{ return F2D; } ; /* Float to decimal */
getenv			{ return GETENV; } ; /* Get environment variable */
setenv			{ return SETENV; } ; /* Set environment variable */
signal			{ return SIGNAL; } ; /* Catch signal */
rsig				{ return RSIG; } ; /* Restore signal action */
chr				{ return CHR; } ; /* int->chr */
system			{ return SYSTEM; } ; /* Run with /bin/sh */
kill				{ return KILL; } ; /* Send signal */
asc				{ return ASC; } ; /* chr->int */
setc				{ return SETC; } ; /* Set character in string */
unless			{ return SKIP; } ; /* Skip block */
in					{ return IN; } ; /* Input number */
drop				{ return DROP; } ; /* Drop number */
din				{ return dIN; } ; /* Direct input from stdin */
prompt			{ return PROMPT; } ; /* Prompt to stderr */
format			{ return FORMAT; } ; /* Format string */
stos				|
sto				{ return STO; } ; /* Store */
gets				|
get				{ return GET; } ; /* Get variable */
pipeic			{ return PIPEIC; } ; /* Open input pipe */
pipeoc			{ return PIPEOC; } ; /* Open output pipe */
pipei				{ return PIPEI; } ; /* Open input pipe */
pipeo				{ return PIPEO; } ; /* Open output pipe */
stat				{ return STAT; } ; /* Do a stat on a file */
chmod				{ return CHMOD; } ; /* Change permissions */
rename			{ return RENAME; } ; /* Rename files */
chdir				{ return CHDIR; } ; /* Change directory */
unlink			{ return UNLINK; } ; /* Remove link to file */
openi				{ return OPENI; } ; /* Open input */
glob				{ return RDIR; } ; /* Read directory */
openo				{ return OPENO; } ; /* Open output */
opena				{ return OPENA; } ; /* Open to append */
closeo			|
restoreo			{ return RESTOREO; } ; /* Restore output to stdout */
closei			|
restorei			{ return RESTOREI; } ; /* Restore input to stdin */
split				{ return SPLIT; } ; /* Split string by delimiter */
s.?				{ return S; } ; /* Single character regexp ops: substitute */
m.?				{ return M; } ; /* Match */
p					{ return P; } ; /* Split */
mre				{ return MRE; } ; /* Match regexp */
sre				{ return SRE; } ; /* Set regexp */
trd				{ return TRD; } ; /* Delete characters from string */
lenS				{ return sLEN; } ; /* Get number of characters in string */
substr			{ return SUBSTR; } ; /* Like perl substr */
rightS			{ return sRIGHT; } ; /* Get chars from right side of string */
posS				{ return sPOS; } ; /* Position of string */
subS				{ return sSUB; } ; /* Substitute string */
leftS				{ return sLEFT; } ; /* Get chars from left side of string */
reverseS			{ return REVERSE; } ; /* Reverse a string */
tr					{ return TR; } ; /* Translate letters */
snext				{ return SNEXT; } ; /* Set next level of stack to act on */
dup				{ return DUP; } ; /* Duplicate top level of number stack */
dup2				{ return DUP2; } ; /* Double DUP */
pick				{ return PICK; } ; /* Pick from number stack */
swap				{ return SWAP; } ; /* Swap x and y */
depth				{ return DEPTH; } ; /* Return depth of nums stack */
depthS			{ return sDEPTH; } ; /* Return depth of strings stack */
dropS				{ return sDROP; } ; /* Drop a string */
dupS				{ return sDUP; } ; /* Duplicate a string */
pickS				{ return sPICK; } ; /* Pick a string */
ucS				{ return UC; } ; /* Uppercase */
lcS				{ return LC; } ; /* Lower case */
swapS				{ return sSWAP; } ; /* Swap x and y (string) */
end				{ return END; } ; /* End execution of program */
waiting			{ return WAITING; } ; /* Get # of waiting characters */
inS				{ return sIN; } ; /* Input string */
getc				{ return GETC; } ; /* Get character */
EOF				{ return NEOF; } ; /* Return an EOF marker */
outS				{ return sOUT; } ; /* Output string */
write				{ return FOUT; } ; /* Formatted output */
chompS			{ return sCHOMP; } ; /* Chop newline off string */
catS				{ return sCAT; } ; /* Concatenate string */
\$					|
\->str			{ return tSTR; } ; /* Number->string */
\;					|
\->num			{ return tNUM; } ; /* String->number */
\?					|
\->float			{ return tFLOAT; } ; /* String->float */
\->ll				{ return tLIST; } ; /* Stack->list */
AND				{ return AND; } ; /* Boolean AND */
OR					{ return OR; } ; /* Boolean OR */
NOT				{ return NOT; } ; /* Boolean NOT */
\&					{ return BAND; } ; /* These are same as */
\|					{ return BOR; } ; /* above, but work as */
~					{ return BNOT; } ; /* bitwise operators */
EQ					|
\=					{ return EQUALS; }; /* Numerical equals */
seq				|
SEQ				{ return SEQ; } ; /* String equals */
\>					{ return MORE; }; /* Numerical > */
\<					{ return LESS; }; /* Numerical < */
!=					{ return NEQ; }; /* Numerical =/= */
\n					{ lcount++; }; /* New line on preprocessor'd stuff */
[ \t]+			;
[^\t\n ]+		{ if(strlen(yytext) == 3 && yytext[0] == '\'') 
					  { spare = (char *)mmalloc(strlen(yytext) + 3); sprintf(spare,
						 "%d", yytext[1]); yytext = spare; return NUM; }
					  spare = (char *)mmalloc(strlen(yytext) + 3); sprintf(spare,
					  "&%s", yytext); yytext = spare; return SUBC;
					};
%%

